import asses_results
import automatic_prediciton
import matplotlib.pyplot as plt
from pathlib import Path
import numpy as np

def collect_data():
    answers = asses_results.get_all_data_from_folder("results", "csv")
    headers = answers[0]
    only_answers = answers[1:]

    survey_data = asses_results.process_survey_data(headers, only_answers)
    
    # correct answers for each survey group (LLM generated text is the correct answer)
    correct_text = ["Text 2", "Text 2", "Text 1", "Text 2", "Text 1", "Text 2",
                    "Text 2", "Text 2", "Text 1", "Text 1", "Text 2", "Text 1",
                    "Text 1", "Text 2", "Text 2", "Text 1", "Text 1", "Text 2",]
    
    models = ["gpt2", "gpt2", "opt", "opt", "gpt4o", "gpt4o",
              "gpt4o", "gpt4o", "opt", "opt", "gpt2", "gpt2",
              "opt", "opt", "gpt4o", "gpt4o", "gpt2", "gpt2"]
    
    evaluator = asses_results.Proccess_Data(correct_text, survey_data, models)
    total_correct = evaluator.calculate_correct_answers()
    total_time = evaluator.get_time()
    
    expert_group = evaluator.compare_groups("expert")
    ai_usage_group = evaluator.compare_groups("ai_usage")
    time_group = evaluator.compare_groups("time")
    
    correct_percentage = [i[2] for i in total_correct.values()] # extracts average percentage of correct answers
    model_results = evaluator.compare_ai(correct_percentage)

    parameters = ["Coherence", "Conciseness", "Creativity", "Clarity of Concept"]
    models = ["gpt2", "gpt4o", "opt", "human"]
    
    avg_scores_model = {
    param: {model: evaluator.average_parameter_model(param, model) for model in models}
    for param in parameters
    }
    
    avg_scores_question = {param: evaluator.parameter_survey_group_assesment(param) for param in parameters}
    
    return total_correct, expert_group, ai_usage_group, time_group, model_results, avg_scores_model, total_time, avg_scores_question

def save_bar_chart(categories, values, filename="barchart.png", title="Average Correct Guess", y="percentage"):
    """Creates and saves a bar chart as a PNG file in the results folder."""
    script_dir = Path(__file__).resolve().parent
    results_dir = script_dir.parent / "results"

    # Create bar chart
    plt.figure(figsize=(8, 6))
    plt.bar(categories, values, color='steelblue')
    if y == "percentage":
        # Set y-axis limits from 0 to 1
        plt.ylim(0, 1)

        # Format y-axis as percentages
        plt.gca().set_yticks([i / 10 for i in range(11)])  # 0.0, 0.1, ..., 1.0
        plt.gca().set_yticklabels([f"{int(y*100)}%" for y in plt.gca().get_yticks()])
        plt.ylabel('Percentage')
    if y == "int":
        max_value = max(values)
        max_y = max_value + 1
        # Set y-axis limits from 0 to max_y
        plt.ylim(0, max_y)

        # Set y-ticks from 0 to max_y with a step of 10
        plt.gca().set_yticks(range(0, int(max_y) + 1, 10))  # Step of 10
        plt.ylabel('Time (in minutes)')
        
    # Labels and title
    plt.xlabel('Categories')
    
    plt.title(title)

    # Save plot as a PNG file
    save_path = results_dir / filename
    plt.savefig(save_path, bbox_inches='tight')
    plt.close()  # Close the plot to free memory

    return save_path

def save_grouped_bar_chart(results, filename="param_scores.png"):
    """Creates and saves a grouped bar chart with model names inside the bars."""
    script_dir = Path(__file__).resolve().parent
    results_dir = script_dir.parent / "results"

    parameters = list(results.keys())  # Extract parameter names
    model_names = list(next(iter(results.values())).keys())  # Extract model names

    # Convert dictionary into a 2D list (rows: parameters, columns: models)
    values = []
    for param in parameters:
        param_values = []
        for model in model_names:
            param_values.append(results[param][model])
        values.append(param_values)

    values = np.array(values)

    # Plot settings
    x = np.arange(len(parameters))  # X positions for groups
    width = 0.2  # Width of bars

    plt.figure(figsize=(10, 6))

    # Create bars for each model
    for i, model in enumerate(model_names):
        bars = plt.bar(x + i * width - (width * (len(model_names) - 1) / 2), values[:, i], width, label=model)

        # Add model name inside each bar
        for bar in bars:
            height = bar.get_height()
            plt.text(bar.get_x() + bar.get_width() / 2, height / 2, model, ha='center', va='center', fontsize=10, color='white', fontweight='bold')

    # Labels and styling
    plt.xlabel("Evaluation Parameters")
    plt.ylabel("Scores")
    plt.title("Model Performance Across Evaluation Parameters")
    plt.xticks(x, parameters)
    plt.ylim(0, 5)  # Assuming scores range between 0 and 5
    plt.legend(title="Models")

    # Save the figure
    save_path = results_dir / filename
    plt.savefig(save_path, bbox_inches='tight')
    plt.close()  # Close plot to free memory

    return save_path

if __name__ == '__main__':
    results = collect_data()

    # Average correct guess for all the survey groups across different participant groups
    categories = ['Average correct', 'Experts', 'Freq. AI usage', 'Below avg Time', 'Above avg Time']
    values = [results[0][0][2], results[1][0][2], results[2][0][2], results[3][0][0][2], results[3][1][0][2]]
    save_bar_chart(categories, values, "correct_guess.png")

    # Average correct guess across all survey groups
    categories = [str(num) for num in list(results[0].keys())]
    values = [total[2] for total in list(results[0].values())]
    save_bar_chart(categories, values, "survey_groups_correct.png")

    # Time spent with survey
    categories = [str(participant) for participant in list(results[6][1].keys())]
    categories.append("0")
    values = [time_spent for time_spent in list(results[6][1].values())]
    values.append(results[6][0])
    save_bar_chart(categories, values, "time_spent.png", "Time Spent", y="int")

    # Average scores for each parameter across models
    save_grouped_bar_chart(results[5])

    # Correct Guess for each model
    categories = list(results[4].keys())
    values = list(results[4].values())
    save_bar_chart(categories, values, "model_results.png")

    predictions = automatic_prediciton.calculated_predictions()

    # Show how often the simple model predicts correctly
    predictions_tag = predictions[0]
    categories = [str(question_num) for question_num in predictions_tag.keys()]
    values = []
    for tag in predictions_tag.values():
        if tag == "equal":
            values.append(0)
        if tag == "human":
            values.append(1)
    save_bar_chart(categories, values, "automated_prediction.png", "Automated Predictiion", y="int")

    # counts how often the model rates the parameters higher for ai or human compared to survey results from humans
    predicitons_param = predictions[1]
    human_param = results[7]
    count, count_total = 0, 0
    for question_num, answers in enumerate(predicitons_param.values()):
        for param, predicted_tag in answers.items():
            try:
                human_response = human_param[param][question_num]
                if predicted_tag == human_response:
                    count += 1
                else:
                    count_total += 1
            except:
                count_total += 1

    match_accuracy = count / count_total