from transformers import pipeline, set_seed
import os

poem_strings = ["write a beautiful and creative poem about nature:",
                "write a thoughtful poem about existance and life:",
                "write a poem about my dog:"
]
wiki_strings = ["Explain what Quantum Tunneling is.", "Explain what Extremophiles are.",
                     "Explain what Plate Tectonics is.","Explain what Bose-Einstein Condensate is.",
                    "Explain what Ferrofluids are.", "Explain what Dark Matter is."
]
sport_strings = [
    "This is a summary of an exciting football match:",
    "This is a summary of a an intense footbal game between two rival teams:",
    "This is an analysis of a football game:",
]



def write_file(output_directory, filename, text: dict):
    file = os.path.join(output_directory, f"{filename}.txt")
    
    with open(file, 'w', encoding='utf-8') as file:
        file.write(f"{filename} Extracts:\n\n")
        for title, content in text.items():
            file.write(f"Input prompt: {title}\n")
            for idx, output in enumerate(content):
                file.write(f"{idx}: {output}\n")
            file.write("-----\n\n\n")

if __name__ == '__main__':
    
    test_string = "Let's talk about the wonders of space exploration and how it's changing the way we perceive the universe."
    set_seed(42)
    
    gpt2_generator = pipeline('text-generation', model='gpt2')
    opt_generator = pipeline('text-generation', model="facebook/opt-125m", do_sample=True)
    
    # POEM
    result_gpt2_poem = {}
    for poem in poem_strings:
        generated_outputs_gpt = gpt2_generator(poem,truncation=True, max_length=150,  num_return_sequences=3, temperature=0.8, top_p=0.75, top_k=40,repetition_penalty=1.7, eos_token_id=50256)
        gpt2_outputs = [output['generated_text'] for output in generated_outputs_gpt]
        result_gpt2_poem[poem] = gpt2_outputs
    
    result_opt_poem = {}
    for poem in poem_strings:
        generated_outputs_opt = opt_generator(poem,truncation=True, max_length=150,num_return_sequences=3, temperature=0.8, top_p=0.75, top_k=40,repetition_penalty=1.7, eos_token_id=50256)
        opt_outputs = [output['generated_text'] for output in generated_outputs_opt]
        result_opt_poem[poem] = opt_outputs
    
    # WIKI
    result_gpt2_wiki = {}
    for topic in wiki_strings[:3]:
        generated_outputs_gpt = gpt2_generator(topic,truncation=True, max_length=140,  num_return_sequences=3, temperature=0.5, top_p=0.4, top_k=30,repetition_penalty=1.7, eos_token_id=50256)
        gpt2_outputs = [output['generated_text'] for output in generated_outputs_gpt]
        result_gpt2_wiki[topic] = gpt2_outputs
    
    result_opt_wiki = {}
    for topic in wiki_strings[3:]:
        generated_outputs_opt = opt_generator(topic,truncation=True, max_length=140,num_return_sequences=3, temperature=0.5, top_p=0.4, top_k=30,repetition_penalty=1.7, eos_token_id=50256)
        opt_outputs = [output['generated_text'] for output in generated_outputs_opt]
        result_opt_wiki[topic] = opt_outputs
    
    # SPORTS
    result_gpt2_sports = {}
    for topic in sport_strings:
        generated_outputs_gpt = gpt2_generator(topic,truncation=True, max_length=140,  num_return_sequences=3, temperature=0.5, top_p=0.4, top_k=30,repetition_penalty=1.7, eos_token_id=50256)
        gpt2_outputs = [output['generated_text'] for output in generated_outputs_gpt]
        result_gpt2_sports[topic] = gpt2_outputs
    
    result_opt_sports = {}
    for topic in sport_strings:
        generated_outputs_opt = opt_generator(topic,truncation=True, max_length=140,num_return_sequences=3, temperature=0.5, top_p=0.4, top_k=30,repetition_penalty=1.7, eos_token_id=50256)
        opt_outputs = [output['generated_text'] for output in generated_outputs_opt]
        result_opt_sports[topic] = opt_outputs
    
    # output_directory = r"C:\Users"
    # write_file(output_directory, "gpt2_poem", result_gpt2_poem)
    # write_file(output_directory, "opt_poem_new", result_opt_poem)
    # write_file(output_directory, "gpt2_wiki", result_gpt2_wiki)
    # write_file(output_directory, "opt_wiki", result_opt_wiki)
    # write_file(output_directory, "gpt2_sport", result_gpt2_sports)
    # write_file(output_directory, "opt_sport", result_opt_sports)
