import kagglehub
import re
import csv
import wikipediaapi
import os
from time import sleep
from bs4 import BeautifulSoup
import requests

def get_first_n_sentences(text, n=6):
    # Split the content by periods to separate sentences
    sentences = re.split(r"(?<=[.?!])\s+", text)
    
    result = ' '.join(sentences[:n]).strip() if len(sentences) >= n else text
    
    return result

def extract_pf(amount: str, sentence_flag: bool):

    path = kagglehub.dataset_download("tgdivy/poetry-foundation-poems")
    path = path + "\\PoetryFoundationData.csv"

    poetry_data = {}
    cleaned_poetry_data = {}
    with open(path, 'r', encoding='utf-8') as file:
        reader = csv.reader(file)
        next(reader)

        for row in reader:
            if not row or len(row) < 3:
                continue

            key = row[0].strip()
            if key == amount:
                break

            content = row[2].strip()
            
            if content.startswith('"') and content.endswith('"'):
                content = content[1:-1]

            cleaned_content = ' '.join(content.split())
            if sentence_flag:
                first_six_sentences = get_first_n_sentences(content, n=6)
                first_six_sentences_cl = get_first_n_sentences(cleaned_content, n=6)
                
                poetry_data[key] = first_six_sentences
                cleaned_poetry_data[key] = first_six_sentences_cl
            else:
                poetry_data[key] = content
                cleaned_poetry_data[key] = cleaned_content
    return poetry_data, cleaned_poetry_data

def get_wikipedia_text(page_titles: list):
    wiki = wikipediaapi.Wikipedia(
        language='en', 
        user_agent='LanguageEvalBot/0.1 (fiwen4@gmail.com)'
    )
    
    wiki_texts = {}
    for title in page_titles:
        page = wiki.page(title)
        extracted_text = page.text
        sleep(0.5)
        if extracted_text:
            first_three_sentences = get_first_n_sentences(extracted_text, 3)
            wiki_texts[title] = first_three_sentences
    return wiki_texts

def get_bbcsport_text(page_title):
    sports_text = {}
    for title in page_title:
        response = requests.get(title)
        sleep(0.5)
        response.text
        soup = BeautifulSoup(response.text, 'html.parser')
        article = soup.find('article')
        if article:
            paragraphs = article.find_all('p')
            content = paragraphs[1:] # cuts out the title
            uncut_sport_text = " ".join([p.get_text() for p in content])
            sports_text[paragraphs[0].get_text()] = get_first_n_sentences(uncut_sport_text, 3)
        else:
                print(f"No paragraphs found in the article for {title}.")
    return sports_text
        
def write_file(output_directory, filename, text: dict):
    file = os.path.join(output_directory, f"{filename}.txt")
    
    with open(file, 'w', encoding='utf-8') as file:
        file.write(f"{filename} Extracts:\n\n")
        for title, content in text.items():
            file.write(f"{title}\n{content}\n---\n")
    

if __name__ == "__main__":
    wiki_page_titles = [
    "Schrödinger's_cat", "Entropy", "Theory_of_relativity",
    "DNA_replication", "Artificial_intelligence", "Global_warming",
    "Evolutionary_biology", "Quantum_entanglement", "Blockchain"
]
    sport_urls = [
        "https://www.bbc.com/sport/football/live/c2kgnl0n9v4t","https://www.bbc.com/sport/football/live/c0rz98kkl0yt",
        "https://www.bbc.com/sport/football/live/cdx2j02n5ljt", "https://www.bbc.com/sport/football/articles/cd0j98vy985o",
        "https://www.bbc.com/sport/football/live/cy4vrjnrw4gt", "https://www.bbc.com/sport/football/live/c62xndp2rejt",
        "https://www.bbc.com/sport/football/live/c9de9y7d13dt", "https://www.bbc.com/sport/football/articles/cn482d222d7o",
        "https://www.bbc.com/sport/football/articles/cjw2xxq9yy1o"
        
    ]
    
    poetry_text = extract_pf("9", True)
    wiki_text = get_wikipedia_text(wiki_page_titles)
    sports_text = get_bbcsport_text(sport_urls)
    
    # output_directory = r"C:\Users"
    
    # write_file(output_directory, "poetry_newlines", poetry_text[0])
    # write_file(output_directory, "wiki", wiki_text)
    # write_file(output_directory, "sport_bbc", sports_text)
    
